import processing.core.*; import java.applet.*; import java.awt.*; import java.awt.image.*; import java.awt.event.*; import java.io.*; import java.net.*; import java.text.*; import java.util.*; import java.util.zip.*; public class Entrapment extends PApplet {// ------------------------------------------------------------------------------
// Entrapment - http://www.arkitus.com
// Copyright (C) 2005  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Entrapment.java
// Procedures related to starting up the program and adjusting the settings
// are in this file.
// ------------------------------------------------------------------------------

int num_u        = 30;                 // Number of units
int num_o        = 60;                 // Number of obstacles
int counter      = 0;                  // Counter for creating lines
float unit_r     = 5;                  // Radius of each unit
float unit_vmax  = 4;                  // Maximum speed of each unit
float unit_smax  = 0.6f;                // Maximum steer of each unit
float unit_wand  = 80;                 // Constant for wander speed
float unit_sdmp  = 0.4f;                // Steer auto align to center
float unit_tfut  = 15;                 // Number of frames in the future units check for collisions
float unit_prox  = 4*unit_r;           // Distance to check for proximity collisions
boolean draw_vel = true;               // Draw velocity line?
boolean draw_str = false;              // Draw steer line?
boolean draw_prj = false;              // Draw projections?
boolean wander   = true;               // Wander?

Unit[] Units;                          // Create array for units
Obstacle[] Obstacles;                  // Create array for obstacles

public void setup()
{
  framerate(25);
  size(800,420);
  reset();
}

public void draw()
{
  update();
  drawStuff();
  drawButton();
}

public void mousePressed()
{
  if ( mouseX > 700 && mouseX < 705 && mouseY > 340 && mouseY < 345 )
  {
    if ( draw_prj )
    {
      draw_prj = false;
    }
    else
    {
      draw_prj = true;
    }
  }
  else
  {
    counter = 0;
  }
}

public void mouseDragged()
{
  if ( counter == 0 )
  {
    for ( int i=counter; i<num_o; i++ )
    {
      Obstacles[i].setPos(mouseX, mouseY);
    }
  }
  if ( counter < num_o )
  {
    Obstacles[counter].setPos(mouseX, mouseY);
    counter++;
  }
}

public void drawButton()
{
  stroke(0,0,0);
  strokeWeight(1);
  if ( draw_prj ) { fill(100,100,100); } else { fill(230,230,230); }
  noSmooth();
  rect(700,340,5,5);
  smooth();
}

public void reset()
{
  Units = new Unit[num_u];
  
  for ( int i=0; i<num_u; i++ )
  {
    Units[i] = new Unit(random(10,width-10), random(10,height-10));
  }
  
  Obstacles = new Obstacle[num_o];
  
  for ( int i=0; i<num_o; i++ )
  {
    Obstacles[i] = new Obstacle();
  }
}

public void update()
{
  for ( int i=0; i<num_u; i++ )
  {
    Units[i].doNotHitWalls();
    Units[i].doNotHitEachOther();
    Units[i].doNotHitObstacles();
    if ( wander ) { Units[i].wander(); }
    Units[i].move();
    Units[i].calculate();
  }
}

public void drawStuff()
{
  background(255,255,255);
  
  for ( int i=0; i<num_u; i++ )
  {
    Units[i].display();
  }
  
  for( int i=0; i<counter; i++ )
  {
    Obstacles[i].display();
  }
  
  for( int i=0; i<counter-1; i++ )
  {
    stroke(150,202,255);
    strokeWeight(1);
    line(Obstacles[i].x,Obstacles[i].y,Obstacles[i+1].x,Obstacles[i+1].y);
  }
}

// ------------------------------------------------------------------------------
// Entrapment - http://www.arkitus.com
// Copyright (C) 2005  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Obstacle.java
// The obstacles the units try to avoid
// ------------------------------------------------------------------------------

class Obstacle
{
  float x;
  float y;
  
  Obstacle()
  {
    x = 0;
    y = 0;
  }
  
  public void setPos(float X, float Y)
  {
    x = X;
    y = Y;
  }
  
  public void display()
  {
    stroke(132,199,255);
    strokeWeight(4);
    point(x,y);
  }
}

// ------------------------------------------------------------------------------
// Entrapment - http://www.arkitus.com
// Copyright (C) 2005  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Unit.java
// The agents that try to avoid the walls, each other and the obstacles.
// ------------------------------------------------------------------------------

class Unit
{
  float x;
  float y;
  float xf;                    // X in future
  float yf;                    // Y in future
  float xr;                    // Proximity Right X
  float yr;                    // Proximity Right Y
  float xl;                    // Proximity Left X
  float yl;                    // Proximity Left Y
  float vd;                    // Size of velocity
  float vt;                    // Rotation of velocity ( relative to the positive x axis )
  float sd;                    // Size of steer
  float st;                    // Rotation of steer ( relative to the velocity )
  
  Unit( float X, float Y )
  {
    x = X;
    y = Y;
    vd = unit_vmax;
    sd = unit_smax;
  }
  
  public void wander()
  {
    st = degrees(st);
    st += random(-unit_wand/2, unit_wand/2);
    st = radians(st);
  }
  
  public void move()
  {
    
    float vx = vd*cos(vt);
    float vy = vd*sin(vt);
    float sx = sd*cos(vt-st);
    float sy = sd*sin(vt-st);
    
    vx += sx;
    vy += sy;
    
    vd = sqrt(pow(vx,2)+pow(vy,2));
    vt = atan2(vy,vx);
    
    st *= unit_sdmp;
    
    truncate();
    
    x += vd*cos(vt);
    y += vd*sin(vt);
  }
  
  public void truncate()
  {
    if ( vd > unit_vmax )
    {
      vd = unit_vmax;
    }
  }
  
  public void calculate()
  {
    xf = x+unit_tfut*vd*cos(vt);
    yf = y+unit_tfut*vd*sin(vt);
    
    xr = x+unit_prox*cos(vt-PI/6);
    yr = y+unit_prox*sin(vt-PI/6);
    
    xl = x+unit_prox*cos(vt+PI/6);
    yl = y+unit_prox*sin(vt+PI/6);
  }
  
  public void doNotHitWalls()
  {
    if ( xf < 0 || xf > width || yf < 0 || yf > height )
    {
      st = PI;
    }
    if ( xr < 0 || xr > width || yr < 0 || yr > height )
    {
      st = -3*PI/4;
    }
    if ( xl < 0 || xl > width || yl < 0 || yl > height )
    {
      st = 3*PI/4;
    }
  }
  
  public void doNotHitEachOther()
  {
    for ( int i=0; i<num_u; i++ )
    {
      float deltax = x - Units[i].x;
      float deltay = y - Units[i].y;
      
      if ( sqrt(pow(deltax,2)+pow(deltay,2)) > unit_r )
      {
        if ( sqrt(pow(xf - Units[i].xf,2)+pow(yf - Units[i].yf,2)) < 4*unit_r )
        {
          st = PI;
        }
        if ( sqrt(pow(xr - Units[i].x,2)+pow(yr - Units[i].y,2)) < 2*unit_r )
        {
          st = -3*PI/4;
        }
        if ( sqrt(pow(xl - Units[i].x,2)+pow(yl - Units[i].y,2)) < 2*unit_r )
        {
          st = 3*PI/4;
        }
      }
    }
  }
  
  public void doNotHitObstacles()
  {
    for ( int i=0; i<num_o; i++ )
    {
      if ( sqrt(pow(xf - Obstacles[i].x,2)+pow(yf - Obstacles[i].y,2)) < unit_prox )
      {
        st = PI;
      }
      if ( sqrt(pow(xr - Obstacles[i].x,2)+pow(yr - Obstacles[i].y,2)) < unit_prox )
      {
        st = -3*PI/4;
      }
      if ( sqrt(pow(xl - Obstacles[i].x,2)+pow(yl - Obstacles[i].y,2)) < unit_prox )
      {
        st = 3*PI/4;
      }
    }
  }
  
  //******************************
  // DRAWING FUNCTIONS
  //******************************
  
  public void display()
  {
    if ( draw_prj ) { drawProjections(); }
    drawBody();
    if ( draw_vel ) { drawVelocity(); }
    if ( draw_str ) { drawSteer(); }
  }
  
  public void drawBody()
  {
    stroke(0,0,0);
    strokeWeight(2*unit_r);
    point(x,y);
  }
  
  public void drawVelocity()
  {
    stroke(255,191,18);
    strokeWeight(2);        
    line(x,y,x+1.5f*unit_r*cos(vt),y+1.5f*unit_r*sin(vt));
  }
  
  public void drawSteer()
  {
    stroke(59,142,255);
    strokeWeight(2);
    line(x,y,x+unit_r*cos(vt-st),y+unit_r*sin(vt-st));
  }
  
  public void drawProjections()
  {
    stroke(255,62,62);
    strokeWeight(3);
    point(xf,yf);
    
    strokeWeight(2);
    stroke(234,234,234);
    line(x,y,xr,yr);
    stroke(59,177,22);
    point(xr,yr);
    
    strokeWeight(2);
    stroke(234,234,234);
    line(x,y,xl,yl);
    stroke(59,177,22);
    point(xl,yl);
  }
}
static public void main(String args[]) {   PApplet.main(new String[] { "Entrapment" });}}