// ------------------------------------------------------------------------------
// Entrapment - http://www.arkitus.com
// Copyright (C) 2005  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Entrapment.java
// Procedures related to starting up the program and adjusting the settings
// are in this file.
// ------------------------------------------------------------------------------

int num_u        = 30;                 // Number of units
int num_o        = 60;                 // Number of obstacles
int counter      = 0;                  // Counter for creating lines
float unit_r     = 5;                  // Radius of each unit
float unit_vmax  = 4;                  // Maximum speed of each unit
float unit_smax  = 0.6;                // Maximum steer of each unit
float unit_wand  = 80;                 // Constant for wander speed
float unit_sdmp  = 0.4;                // Steer auto align to center
float unit_tfut  = 15;                 // Number of frames in the future units check for collisions
float unit_prox  = 4*unit_r;           // Distance to check for proximity collisions
boolean draw_vel = true;               // Draw velocity line?
boolean draw_str = false;              // Draw steer line?
boolean draw_prj = false;              // Draw projections?
boolean wander   = true;               // Wander?

Unit[] Units;                          // Create array for units
Obstacle[] Obstacles;                  // Create array for obstacles

void setup()
{
  framerate(25);
  size(800,420);
  reset();
}

void draw()
{
  update();
  drawStuff();
  drawButton();
}

void mousePressed()
{
  if ( mouseX > 700 && mouseX < 705 && mouseY > 340 && mouseY < 345 )
  {
    if ( draw_prj )
    {
      draw_prj = false;
    }
    else
    {
      draw_prj = true;
    }
  }
  else
  {
    counter = 0;
  }
}

void mouseDragged()
{
  if ( counter == 0 )
  {
    for ( int i=counter; i<num_o; i++ )
    {
      Obstacles[i].setPos(mouseX, mouseY);
    }
  }
  if ( counter < num_o )
  {
    Obstacles[counter].setPos(mouseX, mouseY);
    counter++;
  }
}

void drawButton()
{
  stroke(0,0,0);
  strokeWeight(1);
  if ( draw_prj ) { fill(100,100,100); } else { fill(230,230,230); }
  noSmooth();
  rect(700,340,5,5);
  smooth();
}

void reset()
{
  Units = new Unit[num_u];
  
  for ( int i=0; i<num_u; i++ )
  {
    Units[i] = new Unit(random(10,width-10), random(10,height-10));
  }
  
  Obstacles = new Obstacle[num_o];
  
  for ( int i=0; i<num_o; i++ )
  {
    Obstacles[i] = new Obstacle();
  }
}

void update()
{
  for ( int i=0; i<num_u; i++ )
  {
    Units[i].doNotHitWalls();
    Units[i].doNotHitEachOther();
    Units[i].doNotHitObstacles();
    if ( wander ) { Units[i].wander(); }
    Units[i].move();
    Units[i].calculate();
  }
}

void drawStuff()
{
  background(255,255,255);
  
  for ( int i=0; i<num_u; i++ )
  {
    Units[i].display();
  }
  
  for( int i=0; i<counter; i++ )
  {
    Obstacles[i].display();
  }
  
  for( int i=0; i<counter-1; i++ )
  {
    stroke(150,202,255);
    strokeWeight(1);
    line(Obstacles[i].x,Obstacles[i].y,Obstacles[i+1].x,Obstacles[i+1].y);
  }
}
