// ------------------------------------------------------------------------------
// Entrapment - http://www.arkitus.com
// Copyright (C) 2005  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Unit.java
// The agents that try to avoid the walls, each other and the obstacles.
// ------------------------------------------------------------------------------

class Unit
{
  float x;
  float y;
  float xf;                    // X in future
  float yf;                    // Y in future
  float xr;                    // Proximity Right X
  float yr;                    // Proximity Right Y
  float xl;                    // Proximity Left X
  float yl;                    // Proximity Left Y
  float vd;                    // Size of velocity
  float vt;                    // Rotation of velocity ( relative to the positive x axis )
  float sd;                    // Size of steer
  float st;                    // Rotation of steer ( relative to the velocity )
  
  Unit( float X, float Y )
  {
    x = X;
    y = Y;
    vd = unit_vmax;
    sd = unit_smax;
  }
  
  void wander()
  {
    st = degrees(st);
    st += random(-unit_wand/2, unit_wand/2);
    st = radians(st);
  }
  
  void move()
  {
    
    float vx = vd*cos(vt);
    float vy = vd*sin(vt);
    float sx = sd*cos(vt-st);
    float sy = sd*sin(vt-st);
    
    vx += sx;
    vy += sy;
    
    vd = sqrt(pow(vx,2)+pow(vy,2));
    vt = atan2(vy,vx);
    
    st *= unit_sdmp;
    
    truncate();
    
    x += vd*cos(vt);
    y += vd*sin(vt);
  }
  
  void truncate()
  {
    if ( vd > unit_vmax )
    {
      vd = unit_vmax;
    }
  }
  
  void calculate()
  {
    xf = x+unit_tfut*vd*cos(vt);
    yf = y+unit_tfut*vd*sin(vt);
    
    xr = x+unit_prox*cos(vt-PI/6);
    yr = y+unit_prox*sin(vt-PI/6);
    
    xl = x+unit_prox*cos(vt+PI/6);
    yl = y+unit_prox*sin(vt+PI/6);
  }
  
  void doNotHitWalls()
  {
    if ( xf < 0 || xf > width || yf < 0 || yf > height )
    {
      st = PI;
    }
    if ( xr < 0 || xr > width || yr < 0 || yr > height )
    {
      st = -3*PI/4;
    }
    if ( xl < 0 || xl > width || yl < 0 || yl > height )
    {
      st = 3*PI/4;
    }
  }
  
  void doNotHitEachOther()
  {
    for ( int i=0; i<num_u; i++ )
    {
      float deltax = x - Units[i].x;
      float deltay = y - Units[i].y;
      
      if ( sqrt(pow(deltax,2)+pow(deltay,2)) > unit_r )
      {
        if ( sqrt(pow(xf - Units[i].xf,2)+pow(yf - Units[i].yf,2)) < 4*unit_r )
        {
          st = PI;
        }
        if ( sqrt(pow(xr - Units[i].x,2)+pow(yr - Units[i].y,2)) < 2*unit_r )
        {
          st = -3*PI/4;
        }
        if ( sqrt(pow(xl - Units[i].x,2)+pow(yl - Units[i].y,2)) < 2*unit_r )
        {
          st = 3*PI/4;
        }
      }
    }
  }
  
  void doNotHitObstacles()
  {
    for ( int i=0; i<num_o; i++ )
    {
      if ( sqrt(pow(xf - Obstacles[i].x,2)+pow(yf - Obstacles[i].y,2)) < unit_prox )
      {
        st = PI;
      }
      if ( sqrt(pow(xr - Obstacles[i].x,2)+pow(yr - Obstacles[i].y,2)) < unit_prox )
      {
        st = -3*PI/4;
      }
      if ( sqrt(pow(xl - Obstacles[i].x,2)+pow(yl - Obstacles[i].y,2)) < unit_prox )
      {
        st = 3*PI/4;
      }
    }
  }
  
  //******************************
  // DRAWING FUNCTIONS
  //******************************
  
  void display()
  {
    if ( draw_prj ) { drawProjections(); }
    drawBody();
    if ( draw_vel ) { drawVelocity(); }
    if ( draw_str ) { drawSteer(); }
  }
  
  void drawBody()
  {
    stroke(0,0,0);
    strokeWeight(2*unit_r);
    point(x,y);
  }
  
  void drawVelocity()
  {
    stroke(255,191,18);
    strokeWeight(2);        
    line(x,y,x+1.5*unit_r*cos(vt),y+1.5*unit_r*sin(vt));
  }
  
  void drawSteer()
  {
    stroke(59,142,255);
    strokeWeight(2);
    line(x,y,x+unit_r*cos(vt-st),y+unit_r*sin(vt-st));
  }
  
  void drawProjections()
  {
    stroke(255,62,62);
    strokeWeight(3);
    point(xf,yf);
    
    strokeWeight(2);
    stroke(234,234,234);
    line(x,y,xr,yr);
    stroke(59,177,22);
    point(xr,yr);
    
    strokeWeight(2);
    stroke(234,234,234);
    line(x,y,xl,yl);
    stroke(59,177,22);
    point(xl,yl);
  }
}
